/*LINTLIBRARY*/
/*  guess_write.c

    This code provides a generic write facility.

    Copyright (C) 2002  Richard Gooch

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*

    This file contains the various utility routines for writing data files in
    any format.


    Written by      Richard Gooch   18-MAR-2002

    Last updated by Richard Gooch   18-MAR-2002


*/
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <karma.h>
#include <karma_foreign.h>
#include <karma_dsxfr.h>
#include <karma_ch.h>
#include <karma_a.h>


/*  Declarations of private functions follow  */


/*  Public functions follow  */

/*EXPERIMENTAL_FUNCTION*/
flag foreign_guess_write (CONST char *filename, multi_array *multi_desc, ...)
/*  [SUMMARY] Write a data structure, guessing the output format.
    <filename> The name of the dataset to write.
    <multi_desc> The multi_array descriptor pointer. The routine will find a
    n-dimensional array within the data structure.
    [VARARGS] The optional attributes are given as pairs of attribute-key
    attribute-value pairs. This list must be terminated with FA_FITS_GUESS_END.
    See [<foreign_ATT_GUESS_WRITE>] for a list of defined attributes.
    [RETURNS] TRUE on succes, else FALSE.
*/
{
    va_list argp;
    unsigned int att_key;
    static char function_name[] = "foreign_guess_write";

    va_start (argp, multi_desc);
    /*  Process attributes  */
    while ( ( att_key = va_arg (argp, unsigned int) ) != FA_GUESS_WRITE_END )
    {
	switch (att_key)
	{
	  default:
	    fprintf (stderr, "Unknown attribute key: %u\n", att_key);
	    a_prog_bug (function_name);
	    break;
	}
    }
    va_end (argp);
    if (foreign_guess_format_from_filename (filename) ==
	FOREIGN_FILE_FORMAT_FITS)
    {
	Channel channel;
	flag ok;

	if ( ( channel = ch_open_file (filename, "w") ) == NULL )
	{
	    fprintf (stderr, "Error opening file: \"%s\"\t%s\n",
		     filename, ERRSTRING);
	    return (FALSE);
	}
	ok = foreign_fits_write (channel, multi_desc, FA_FITS_WRITE_END);
	ch_close (channel);
	if (!ok)
	{
	    fprintf (stderr, "%s: error writing FITS file: \"%s\n",
		     function_name, filename);
	    unlink (filename);
	}
	return (ok);
    }
    return dsxfr_put_multi (filename, multi_desc);
}   /*  End Function foreign_guess_write  */
