/*LINTLIBRARY*/
/*  file.c

    This code provides miscellaneous Karma file utility routines.

    Copyright (C) 2002  Richard Gooch

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*

    This file contains miscellaneous file utility routines for Karma.


    Written by      Richard Gooch   3-JUL-2002

    Updated by      Richard Gooch   18-AUG-2002: Replaced sn printf() with
  strlen() and strcat() which is more portable. Yuk.

    Last updated by Richard Gooch   21-NOV-2002: Minor documentation change.


*/
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <karma.h>
#include <karma_r.h>


/*  Private functions  */


/*  Public functions follow  */

/*EXPERIMENTAL_FUNCTION*/
flag r_check_file_variants (CONST char *path, CONST char **extensions)
/*  [SUMMARY] Test for variants of a filename.
    <path> The base pathname to check.
    <extensions> The list of extensions to append to <<path>>. This must be
    terminated with a NULL pointer.
    [RETURNS] TRUE if one of the paths exists and is a file, else FALSE.
*/
{
    for (; *extensions != NULL; ++extensions)
    {
	size_t len_path;
	struct stat statbuf;
	char tmp[STRING_LENGTH];

	len_path = strlen (path);
	if (len_path + strlen (*extensions) + 1 > STRING_LENGTH) continue;
	strncpy (tmp, path, len_path);
	strcpy (tmp + len_path, *extensions);
	if ( (stat (tmp, &statbuf) == 0) && S_ISREG (statbuf.st_mode) )
	    return (TRUE);
    }
    return (FALSE);
}   /*  End Function r_check_file_variants  */
