/*LINTLIBRARY*/
/*  array_addsub.c

    This code provides add/subtract KConstructArrayManager objects.

    Copyright (C) 2004  Richard Gooch

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*  This file contains all routines needed for managing the construction of
    added/subtracted array data.


    Written by      Richard Gooch   31-MAR-2004

    Last updated by Richard Gooch   31-MAR-2004


*/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <karma.h>
#include <k_event_codes.h>
#include <karma_construct.h>
#include <karma_dispdata.h>
#include <karma_iarray.h>
#include <karma_listw.h>
#include <karma_ds.h>
#include <karma_st.h>
#include <karma_m.h>
#include <karma_c.h>
#include <karma_a.h>


#define NUM_RESERVED_CHARS            2
#define MAGIC_NUMBER         2109728011

#define VERIFY_OBJECT(object) {if (object == NULL) \
{fprintf (stderr, "NULL object passed\n"); \
 a_prog_bug (function_name); } \
if (object->magic_number != MAGIC_NUMBER) \
{fprintf (stderr, "Invalid object\n"); \
 a_prog_bug (function_name); } }


struct construct_array_addsub
{
    unsigned int magic_number;
    KConstructArrayManager manager;
    KListWidget listw;
    KDisplayDataClass dataclass;
    unsigned int num_entries;
    KDisplayDataPrivateNameList namelist;
    char **names;
    KDisplayDataObject *dataobjects;
};


/*  Private functions  */
STATIC_FUNCTION (flag _construct_array_addsub_check_pseudo,
		 (void *info, KDisplayDataObject dataobject) );


/*  Private data  */


/* Public functions follow */

/*UNPUBLISHED_FUNCTION*/
void *construct_array_addsub_init (KConstructArrayManager manager,
				   KListWidget list)
{
    struct construct_array_addsub *addsub;
    static char function_name[] = "construct_array_addsub_init";

    if ( ( addsub = m_calloc (sizeof *addsub) ) == NULL )
    {
	m_error_notify (function_name, "ADDSUB manager");
	return (NULL);
    }
    addsub->magic_number = MAGIC_NUMBER;
    addsub->manager = manager;
    addsub->listw = list;
    addsub->dataclass = dispdata_array_create_dataclass ();
    addsub->names = dispdata_regenerate_namelist
	(addsub->dataclass, &addsub->namelist, NUM_RESERVED_CHARS,
	 &addsub->num_entries, &addsub->dataobjects,
	 _construct_array_addsub_check_pseudo, NULL);
    listw_set_attributes (addsub->listw, TRUE,
			  LISTW_ATT_NUM_STRINGS, addsub->num_entries,
			  LISTW_ATT_STRINGS, addsub->names,
			  LISTW_ATT_REVERSE_VIDEO_INDEX, addsub->num_entries,
			  LISTW_ATT_END);
    return (addsub);
}   /*  End Function construct_array_addsub_init  */

/*UNPUBLISHED_FUNCTION*/
void construct_array_addsub_cleanup (void *algorithm_info)
{
    struct construct_array_addsub *addsub = algorithm_info;
    static char function_name[] = "construct_array_addsub_cleanup";

    VERIFY_OBJECT (addsub);
    dispdata_free_namelist (addsub->namelist);
    addsub->magic_number = 0;
    m_free (addsub);
}   /*  End Function construct_array_addsub_cleanup  */

/*UNPUBLISHED_FUNCTION*/
flag construct_array_addsub_position (void *algorithm_info, unsigned int index,
				      unsigned int event_code, void *e_info)
{
    char operation;
    struct construct_array_addsub *addsub = algorithm_info;
    static char function_name[] = "construct_array_addsub_position";

    VERIFY_OBJECT (addsub);
    switch (event_code)
    {
      case K_CANVAS_EVENT_LEFT_MOUSE_RELEASE:
	operation = '+';
	break;
      case K_CANVAS_EVENT_MIDDLE_MOUSE_RELEASE:
	operation = '-';
	break;
      case K_CANVAS_EVENT_RIGHT_MOUSE_RELEASE:
	operation = ' ';
	break;
      case K_CANVAS_EVENT_PLAIN_KEY_PRESS:
	operation = toupper (*(char *) e_info);
	switch (operation)
	{
	  case '+':
	  case '-':
	  case '*':
	  case '/':
	  case ' ':
	    break;
	  default:
	    return (FALSE);
	    /*break;*/
	}
	break;
      default:
	return (FALSE);
	/*break;*/
    }
    addsub->names[index][0] = operation;
    listw_set_attributes (addsub->listw, TRUE,
			  LISTW_ATT_NUM_STRINGS, addsub->num_entries,
			  LISTW_ATT_STRINGS, addsub->names,
			  LISTW_ATT_END);
    return (TRUE);
}   /*  End Function construct_array_addsub_position  */

/*UNPUBLISHED_FUNCTION*/
flag construct_array_addsub_data_create (void *algorithm_info,
					 KDisplayDataObject dataobject)
{
    struct construct_array_addsub *addsub = algorithm_info;
    static char function_name[] = "construct_array_addsub_data_create";

    VERIFY_OBJECT (addsub);
    addsub->names = dispdata_regenerate_namelist
	(addsub->dataclass, &addsub->namelist, NUM_RESERVED_CHARS,
	 &addsub->num_entries, &addsub->dataobjects,
	 _construct_array_addsub_check_pseudo, NULL);
    listw_set_attributes (addsub->listw, TRUE,
			  LISTW_ATT_NUM_STRINGS, addsub->num_entries,
			  LISTW_ATT_STRINGS, addsub->names,
			  LISTW_ATT_REVERSE_VIDEO_INDEX, addsub->num_entries,
			  LISTW_ATT_END);
    return (TRUE);
}   /*  End Function construct_array_addsub_data_create  */

/*UNPUBLISHED_FUNCTION*/
flag construct_array_addsub_data_destroy (void *algorithm_info)
{
    struct construct_array_addsub *addsub = algorithm_info;
    static char function_name[] = "construct_array_addsub_data_destroy";

    VERIFY_OBJECT (addsub);
    addsub->names = dispdata_regenerate_namelist
	(addsub->dataclass, &addsub->namelist, NUM_RESERVED_CHARS,
	 &addsub->num_entries, &addsub->dataobjects,
	 _construct_array_addsub_check_pseudo, NULL);
    listw_set_attributes (addsub->listw, TRUE,
			  LISTW_ATT_NUM_STRINGS, addsub->num_entries,
			  LISTW_ATT_STRINGS, addsub->names,
			  LISTW_ATT_REVERSE_VIDEO_INDEX, addsub->num_entries,
			  LISTW_ATT_END);
    return (TRUE);
}   /*  End Function construct_array_addsub_data_destroy  */

/*UNPUBLISHED_FUNCTION*/
void construct_array_addsub_create (void *algorithm_info, CONST char *name)
{
    iarray arr_first = NULL;
    iarray new;
    char first_operation = ' ';
    unsigned int count;
    multi_array *multi_desc;
    double zero[2], plus_one[2], scale[2], offset[2];
    struct construct_array_addsub *addsub = algorithm_info;
    static char function_name[] = "construct_array_addsub_create";

    VERIFY_OBJECT (addsub);
    /*  First perform sanity checks and find first array (if any)  */
    for (count = 0; count < addsub->num_entries; ++count)
    {
	iarray arr;

	switch (addsub->names[count][0])
	{
	  case '+':
	  case '-':
	  case '*':
	  case '/':
	    dispdata_get_data_attributes (addsub->dataobjects[count],
					  KDISPLAYDATA_DATA_ATT_PSEUDO_ARRAY,
				              &arr,
					  KDISPLAYDATA_DATA_ATT_END);
	    if (arr_first)
	    {
		if ( !iarray_compare_sizes (arr_first, arr, TRUE) ) return;
	    }
	    else
	    {
		arr_first = arr;
		first_operation = addsub->names[count][0];
	    }
	    break;
	}
    }
    if (!arr_first)
    {
	fprintf (stderr, "%s: insufficient data\n", function_name);
	return;
    }
    if ( ( new = iarray_create_from_template2 (arr_first, K_FLOAT,
					       TRUE, TRUE, TRUE, FALSE) )
	 == NULL ) return;
    zero[0] = zero[1] = 0.0;
    plus_one[0] = 1.0;
    plus_one[1] = 0.0;
    iarray_get_data_scaling (arr_first, scale, offset);
    scale[1] = offset[1] = 0.0;
    switch (first_operation)
    {
      case '+':
      case '*':
	if ( (scale[0] == 1.0) && (offset[0] == 0.0) )
	    iarray_copy_data (new, arr_first, FALSE);
	else iarray_scale_and_offset (new, arr_first, scale, offset, FALSE);
	break;
      case '-':
	scale[0] *= -1.0;
	offset[0] *= -1.0;
	iarray_scale_and_offset (new, arr_first, scale, offset, FALSE);
	break;
      case '/':
	iarray_scale_and_offset (new, arr_first, zero, plus_one, FALSE);
	iarray_multiply_divide (new, new, arr_first, FALSE, scale, offset,
				FALSE);
	break;
    }
    /*  Now operate on other arrays  */
    for (count = 0; count < addsub->num_entries; ++count)
    {
	iarray arr;

	dispdata_get_data_attributes (addsub->dataobjects[count],
				      KDISPLAYDATA_DATA_ATT_PSEUDO_ARRAY,
				          &arr,
				      KDISPLAYDATA_DATA_ATT_END);
	if (arr == arr_first) continue;
	iarray_get_data_scaling (arr, scale, offset);
	switch (addsub->names[count][0])
	{
	  case '+':
	    iarray_add_and_scale2 (new, new, arr, scale, offset, FALSE);
	    break;
	  case '-':
	    scale[0] *= -1.0;
	    offset[0] *= -1.0;
	    iarray_add_and_scale2 (new, new, arr, scale, offset, FALSE);
	    break;
	  case '*':
	    iarray_multiply_divide (new, new, arr, TRUE, scale, offset, FALSE);
	    break;
	  case '/':
	    iarray_multiply_divide (new, new, arr, FALSE, scale, offset,FALSE);
	    break;
	  default:
	    continue;
	    /*break;*/
	}
    }
    multi_desc = iarray_multi_array (new);
    count = iarray_multi_index (new);
    /*  Get rid of crud  */
    ds_remove_unique_named_element (multi_desc->headers[count],
				    multi_desc->data + count, "DATAMIN", TRUE);
    ds_remove_unique_named_element (multi_desc->headers[count],
				    multi_desc->data + count, "DATAMAX", TRUE);
    ds_event_dispatch (multi_desc, "ARITHMETIC", name);
    iarray_dealloc (new);
}   /*  End Function construct_array_addsub_create  */


/*  Private functions follow  */

static flag _construct_array_addsub_check_pseudo (void *info,
						  KDisplayDataObject dataobject)
{
    iarray arr;

    dispdata_get_data_attributes (dataobject,
				  KDISPLAYDATA_DATA_ATT_PSEUDO_ARRAY, &arr,
				  KDISPLAYDATA_DATA_ATT_END);
    return (arr ? TRUE : FALSE);
}   /*  End Function _construct_array_addsub_check_pseudo  */
